<?php

namespace Pinpoint\CarouselManager\Block;

use Magento\Framework\View\Element\Template;

class Carousel extends Template implements \Magento\Widget\Block\BlockInterface
{
    protected $carouselId;
    protected $carouselRespository;
    protected $carousel;

    protected $templateFilterModel = 'Magento\Catalog\Model\Template\Filter';
    protected $templateProcessor;
    protected $templateFilterFactory;

    protected $slides = [];

    public function __construct(\Pinpoint\CarouselManager\Model\CarouselRepository $carouselRepository,
                                \Magento\Catalog\Model\Template\Filter\Factory $templateFilterFactory,
                                Template\Context $context, array $data = [])
    {
        $this->templateFilterFactory = $templateFilterFactory;
        $this->carouselRespository = $carouselRepository;

        $this->_template = 'Pinpoint_CarouselManager::carousel.phtml';

        if (isset($data['template']) AND $data['template']) {
            $this->setTemplate($data['template']);
        }

        if (isset($data['carousel_id']) AND $data['carousel_id']) {
            $this->setCarouselId($data['carousel_id']);
        }

        parent::__construct($context, $data);
    }

    public function getCarouselId()
    {
        return $this->carouselId;
    }

    public function setCarouselId($carouselId)
    {
        $this->carouselId = $carouselId;
    }

    public function getCurrentStore()
    {
        return $this->_storeManager->getStore()->getId();
    }

    public function getCarousel()
    {
        if (!$this->carousel) {
            try {

                $carouselId = $this->getCarouselId();
                $carousel = $this->carouselRespository->getById($carouselId);
                $currentStoreId = $this->getCurrentStore();

            } Catch (\Exception $e) {
                //Silently skip
                $this->_logger->critical($e);
                return null;
            }

            $allowStores = $carousel->getStoreIds();
            if ($allowStores AND $currentStoreId AND is_array($allowStores) AND count($allowStores) AND in_array($currentStoreId, $allowStores)) {
                $this->carousel = $carousel;
            }

        }
        return $this->carousel;
    }

    protected function _toHtml()
    {
        if (!$this->getCarousel()) {
            return '';
        }
        return parent::_toHtml();
    }

    protected function _getTemplateProcessor()
    {
        if (null === $this->templateProcessor) {
            $this->templateProcessor = $this->templateFilterFactory->create($this->templateFilterModel);
        }

        return $this->templateProcessor;
    }

    public function isFilterable($attribute)
    {
        return in_array($attribute, ['title', 'description', 'cta', 'link']);
    }

    public function getSlideProp($slide, $attribute, $default = '')
    {
        if (!isset($this->slides[$slide->getId()][$attribute])) {
            $val = $slide->getData($attribute) ? $slide->getData($attribute) : $default;
            if ($this->isFilterable($attribute)) {
                $val = $this->_getTemplateProcessor()->filter($val);
            }

            $this->slides[$slide->getId()][$attribute] = $val;
        }
        return $this->slides[$slide->getId()][$attribute];
    }

    public function getImageUrl($slide)
    {
        if (!isset($this->slides[$slide->getId()]['image'])) {
            $this->slides[$slide->getId()]['image'] = null;
            $imagePath = $this->getSlideProp($slide, 'image', null);
            if ($imagePath) {
                $imageUrl = $this->getBaseUrl() . $imagePath;
                if ($imageUrl) {
                    $this->slides[$slide->getId()]['image'] = $imageUrl;
                }
            }

        }
        return $this->slides[$slide->getId()]['image'];
    }


}