<?php

namespace Pinpoint\CarouselManager\Controller\Adminhtml\Carousel;

use Magento\Backend\App\Action;
use Pinpoint\CarouselManager\Model\Page;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\LocalizedException;

class Duplicate extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Pinpoint_CarouselManager::carousel';

    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @param Action\Context $context
     * @param DataPersistorInterface $dataPersistor
     */
    public function __construct(
        Action\Context $context,
        DataPersistorInterface $dataPersistor
    )
    {
        $this->dataPersistor = $dataPersistor;
        parent::__construct($context);
    }

    public function execute()
    {
        // check if we know what should be deleted
        $id = $this->getRequest()->getParam('object_id');
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        if ($id) {
            $title = "";
            try {
                // init model and delete
                $model = $this->_objectManager->create('Pinpoint\CarouselManager\Model\Carousel');
                $model->load($id);

                $slides = $model->getSlides();
                $model->setData('id', null);
                $model->save();

                //Get loaded slides from old model, duplicate them with new carousel_id
                if ($slides AND is_array($slides) AND count($slides)) {
                    foreach ($slides as $slide) {
                        $slide->setData('id', null);
                        $slide->setData('carousel_id', $model->getId());
                        $slide->save();
                    }
                }

                // display success message
                $this->messageManager->addSuccess(__('You have duplicated the Carousel.'));
                // go to grid
                return $resultRedirect->setPath('*/*/edit', ['carousel_id' => $model->getId()]);
            } catch (\Exception $e) {
                // display error message
                $this->messageManager->addError($e->getMessage());
                // go back to edit form
                return $resultRedirect->setPath('*/*/edit', ['carousel_id' => $id]);
            }
        }
        // display error message
        $this->messageManager->addError(__('We can not find an object to duplicate.'));
        // go to grid
        return $resultRedirect->setPath('*/*/');

    }
}
