<?php

namespace Pinpoint\CarouselManager\Controller\Adminhtml\Carousel;

use Magento\Backend\App\Action;
use Pinpoint\CarouselManager\Model\Page;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\LocalizedException;

class Save extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Pinpoint_CarouselManager::carousel';

    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @param Action\Context $context
     * @param DataPersistorInterface $dataPersistor
     */
    public function __construct(
        Action\Context $context,
        DataPersistorInterface $dataPersistor
    )
    {
        $this->dataPersistor = $dataPersistor;
        parent::__construct($context);
    }

    /**
     * Save action
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        if ($data) {
            if (empty($data['carousel_id'])) {
                $data['carousel_id'] = null;
            }

            $model = $this->_objectManager->create('Pinpoint\CarouselManager\Model\Carousel');

            $id = $this->getRequest()->getParam('carousel_id');
            if ($id) {
                $model->load($id);
            }

            $model->setData($data);

            try {
                $model->save();
                $this->messageManager->addSuccess(__('Carousel successfully saved.'));
                $this->dataPersistor->clear('pinpoint_carouselmanager_carousel');
                if ($this->getRequest()->getParam('back')) {
                    return $resultRedirect->setPath('*/*/edit', ['carousel_id' => $model->getId(), '_current' => true]);
                }
                return $resultRedirect->setPath('*/*/');
            } catch (LocalizedException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addException($e, __('Something went wrong while saving the data.'));
            }

            $this->dataPersistor->set('pinpoint_carouselmanager_carousel', $data);
            return $resultRedirect->setPath('*/*/edit', ['carousel_id' => $this->getRequest()->getParam('pinpoint_carouselmanager_carousel_id')]);
        }
        return $resultRedirect->setPath('*/*/');
    }
}
