<?php

namespace Pinpoint\CarouselManager\Model;
class Carousel extends \Magento\Framework\Model\AbstractModel implements \Pinpoint\CarouselManager\Api\Data\CarouselInterface, \Magento\Framework\DataObject\IdentityInterface
{
    const CACHE_TAG = 'pinpoint_carousel';
    protected $slideFactory;
    protected $slideRespository;
    protected $searchCriteriaBuilder;
    protected $sortOrderBuilder;
    protected $slides;

    public function __construct(\Magento\Framework\Model\Context $context,
                                \Magento\Framework\Registry $registry,
                                \Pinpoint\CarouselManager\Model\SlideFactory $slideFactory,
                                \Pinpoint\CarouselManager\Model\SlideRepository $slideRespository,
                                \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
                                \Magento\Framework\Api\SortOrderBuilder $sortOrderBuilder,
                                \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
                                \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
                                array $data = [])
    {
        $this->slideFactory = $slideFactory;
        $this->slideRespository = $slideRespository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->sortOrderBuilder = $sortOrderBuilder;

        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    protected function _construct()
    {
        $this->_init('Pinpoint\CarouselManager\Model\ResourceModel\Carousel');
    }

    public function getIdentities()
    {
        return [self::CACHE_TAG . '_' . $this->getId()];
    }

    public function beforeSave()
    {
        $storeIds = $this->getStoreIds();
        if ($storeIds AND is_array($storeIds)) {
            $storeIds = implode(',', $storeIds);
            $this->setStoreIds($storeIds);
        }
        return parent::beforeSave();
    }

    public function afterLoad()
    {
        if ($storeIds = $this->getStoreIds()) {
            $this->setStoreIds(explode(',', $storeIds));
        }
        return parent::afterLoad();
    }


    public function afterSave()
    {
        $data = $this->getData();
        $slidesData = isset($data['slide']) ? $data['slide'] : null;
        if ($slidesData) {
            foreach ($slidesData as $key => $item) {
                if ($key === '%s') {
                    continue;
                }

                $model = $this->slideFactory->create();
                $id = isset($item['id']) ? $item['id'] : 0;

                if (!empty($id)) {
                    $model->load($id);
                }

                if (isset($item['imageUrl'])) {
                    $item['image'] = $item['imageUrl'];
                }

                $model->setData($item);
                $model->setCarouselId($this->getId());
                if (empty($model->getId())) {
                    $model->setId(null);
                }
                try {
                    $model->save();
                } Catch (\Exception $e) {
                    echo 'Exception';
                }

            }
        }

        return parent::afterSave();
    }

    public function getSlides()
    {
        if (!$this->slides AND $this->getId()) {

            $sortOrder = $this->sortOrderBuilder
                ->setField('sort')
                ->setDirection(\Magento\Framework\Api\SortOrder::SORT_ASC)
                ->create();

            $searchCriteria = $this->searchCriteriaBuilder
                ->addFilter('carousel_id', $this->getId(), 'eq')
                ->addSortOrder($sortOrder)
                ->create();

            $this->slides = $this->slideRespository->getList($searchCriteria)->getItems();
        }
        return $this->slides;
    }

}
